//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Dulcimer Tabs Plugin
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  Most of this code was borrowed from various plugins by Nicolas Froment (lasconic)
//  Blame any problems on me. 
//
//  This Duclimer tab plugin is by Keith Onstad in March of 2017
//
//  Plugin tested with version 2.03 of MuseScore -- may work with other versions
// 
//=============================================================================

import QtQuick 2.2
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import MuseScore 3.0

MuseScore {
    version: "2.0"
    description: "Dulcimer Tab plugin"
    menuPath: "Plugins.Dulcimer Tablature"
    pluginType: "dialog"
    
    id: window
    width:280
    height: 180
    ColumnLayout {
        id: column
        anchors.margins : 10
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: 90
        
		// Note -- the mode is not currently used
		//         may add it back in later 
        // ComboBox {
            // currentIndex: 1
            // model: ListModel {
                // id: modeList
                // property var mode
                // ListElement { text: "Mixolydian"; mode: 0 }
                // ListElement { text: "Aeolian"; mode: 1 }
                // ListElement { text: "Locrain"; mode: 2 }
                // ListElement { text: "Ionian"; mode: 3 }
                // ListElement { text: "Dorian"; mode: 4 }
                // ListElement { text: "Phrygian"; mode: 5 }
                // ListElement { text: "Lydian"; mode: 6 }
            // }
            // width: 100
            // onCurrentIndexChanged: {
            // }
        // }
		
		// Note -- Trust me the open note numbers are right it is based on the mod of the pitch
		
        ComboBox {
            currentIndex: 0
            model: ListModel {
                id: noterStringList
                property var openNote
                ListElement { text: "A"; openNote: 8 }
                ListElement { text: "A#/Bb"; openNote: 9 }
                ListElement { text: "B"; openNote: 10 }
                ListElement { text: "C"; openNote: 11 }
                ListElement { text: "C#/Db"; openNote: 0 }
                ListElement { text: "D"; openNote: 1 }
                ListElement { text: "D#/Eb"; openNote: 2 }
                ListElement { text: "E"; openNote: 3 }
                ListElement { text: "F"; openNote: 4 }
                ListElement { text: "F#/Gb"; openNote: 5 }
                ListElement { text: "G"; openNote: 6 }
                ListElement { text: "G#/Ab"; openNote: 7 }
            }
            width: 100
            onCurrentIndexChanged: {
                console.debug(noterStringList.get(currentIndex).text + ", " + noterStringList.get(currentIndex).openNote)
                noterStringList.openNote = noterStringList.get(currentIndex).openNote
            }
        }
		// Note -- the key is not currently used
		// Note -- may add it back in later
		//
        // ComboBox {
            // currentIndex: 1
            // model: ListModel {
                // id: keyList
                // property var selectedKey
                // ListElement { text: "A"; selectedKey: 0 }
                // ListElement { text: "A#"; selectedKey: 1 }
                // ListElement { text: "Bb"; selectedKey: 2 }
                // ListElement { text: "B"; selectedKey: 3 }
                // ListElement { text: "C"; selectedKey: 4 }
                // ListElement { text: "C#"; selectedKey: 5 }
                // ListElement { text: "Db"; selectedKey: 6 }
                // ListElement { text: "D"; selectedKey: 7 }
                // ListElement { text: "D#"; selectedKey: 8 }
                // ListElement { text: "Eb"; selectedKey: 9 }
                // ListElement { text: "E"; selectedKey: 10 }
                // ListElement { text: "F"; selectedKey: 11}
                // ListElement { text: "F#"; selectedKey: 12 }
                // ListElement { text: "Gb"; selectedKey: 13 }
                // ListElement { text: "G"; selectedKey: 14 }
                // ListElement { text: "G#"; selectedKey: 15 }
                // ListElement { text: "Ab"; selectedKey: 16 }
            // }
            // width: 100
            // onCurrentIndexChanged: {
            // }
        // }
        ComboBox {
            currentIndex: 2
            model: ListModel {
                id: placetext
                property var position
                ListElement { text: "Higher"; position: -2 }
                ListElement { text: "Above staff"; position: 0 }
                ListElement { text: "Below staff"; position: 10 }
                ListElement { text: "Lower"; position: 12 }
            }
            width: 100
            onCurrentIndexChanged: {
                console.debug(placetext.get(currentIndex).text + ", " + placetext.get(currentIndex).position)
                placetext.position = placetext.get(currentIndex).position
            }
        }

    }
    RowLayout {
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: column.bottom
        height: 70
        Button {
            id: okButton
            text: "Ok"
            onClicked: {
                apply()
                Qt.quit()
            }
        }
        Button {
            id: closeButton
            text: "Close"
            onClicked: { Qt.quit() }
        }
        
    }
    
    function tabNotes(notes, text) {
        
		var fretBoard = ["0",  "X", "1",  "X", "2",  "3",  "X", "4",  "X", "5",  "6",  "6+",
		                 "7",  "X", "8",  "X", "9",  "10", "X", "11", "X", "12", "13", "13+",
						 "14", "X", "15", "X", "16", "17", "X", "18", "X", "19", "20", "20+",
						 "X",  "X", "X",  "X", "X",  "X",  "X", "X",  "X", "X",  "X",  "X",
						 "X",  "X", "X",  "X", "X",  "X",  "X", "X",  "X", "X",  "X",  "X"]

        
        for (var i = 0; i < notes.length; i++) {
            var sep = "\n"; // change to "," if you want them horizontally
            if ( i > 0 )
                text.text = sep + text.text;
            
            if (typeof notes[i].pitch === "undefined") // just in case
                return
                if (typeof tuning[notes[i].pitch - harpkey] === "undefined")
                    text.text = "X";
                else
                    text.text = tuning[notes[i].pitch - harpkey] + text.text;
        }
    }
    
    function applyToSelection(func) {
        if (typeof curScore === 'undefined')
            Qt.quit();
        var cursor = curScore.newCursor();
        var startStaff;
        var endStaff;
        var endTick;
        var fullScore = false;
        var textposition = placetext.position
		var lowPitch = 9999;
		var startPitch;
        var textposition = placetext.position
        console.log("textposition set to " +placetext.position)
		
		var fretBoard = ["0",  "X", "1",  "X", "2",  "3",  "X", "4",  "X", "5",  "6",  "6+",
		                 "7",  "X", "8",  "X", "9",  "10", "X", "11", "X", "12", "13", "13+",
						 "14", "X", "15", "X", "16", "17", "X", "18", "X", "19", "20", "20+",
						 "X",  "X", "X",  "X", "X",  "X",  "X", "X",  "X", "X",  "X",  "X",
						 "X",  "X", "X",  "X", "X",  "X",  "X", "X",  "X", "X",  "X",  "X"]
		
        console.log("textposition set to " +placetext.position)
		// First we find the lowest note in the score
		
		cursor.rewind(0); // beginning of score
		while (cursor.segment) {
			if (cursor.element && cursor.element.type == Element.CHORD) {
				var notes = cursor.element.notes;
				for (var i = 0; i < notes.length; i++) {
					if (typeof notes[i].pitch === "undefined") // just in case
						return

					if (notes[i].pitch < lowPitch) {
						lowPitch = notes[i].pitch;
					}
				}
			}
			cursor.next();
		}
        console.log("lowPitch = " + lowPitch)

		// Now we calculate the pitch of the noter string
		// Basically, we take the lowest pitch in the score and find the 
		// pitch of the open note we selected that is equal or lower
		
		var lowPitchMod = lowPitch % 12;
        var selectedKey = noterStringList.openNote
		
		if (selectedKey >= lowPitchMod) {
			startPitch = lowPitch - lowPitchMod + selectedKey - 12;
		} else {
			startPitch = lowPitch - lowPitchMod + selectedKey;
		}
        console.log("startPitch = " + startPitch)
        console.log("lowPitchMod = " + lowPitchMod)
        console.log("selectedKey = " + selectedKey)
		
		// Now we can add the tab

		cursor.rewind(0); // beginning of score
		while (cursor.segment) {
			if (cursor.element && cursor.element.type == Element.CHORD) {
				var text = newElement(Element.STAFF_TEXT);
				var notes = cursor.element.notes;
				text.offsetY = textposition;
				for (var i = 0; i < notes.length; i++) {
					var sep = "\n"; // change to "," if you want them horizontally
					if ( i > 0 )
						text.text = sep + text.text;
					
					if (typeof notes[i].pitch === "undefined") // just in case
						return
						if (typeof fretBoard[notes[i].pitch - startPitch] === "undefined")
							text.text = "X";
						else
							text.text = fretBoard[notes[i].pitch - startPitch - 1] + text.text;
//							text.text = notes[i].pitch - startPitch + text.text;
				}
                            cursor.add(text);
			}
			cursor.next();
		}
		
        Qt.quit();
    } // end applyToSelection()
    
    function apply() {
        curScore.startCmd()
        applyToSelection(tabNotes)
        curScore.endCmd()          
    }
    
    onRun: {
        if (typeof curScore === 'undefined')
            Qt.quit();
    }
}
